function IFSfractal = gen_ifs_fractal(pts,trns,iter)
%%GEN_IFS_FRACTAL: Generates iterated-function-system (IFS) collage
%   
% Syntax:
%     IFSfractal = gen_ifs_fractal(base_object,transformations,iteration)
% 
% Examples:
%     IFS1 = gen_ifs_fractal(pt1,tr1,2)
%     where:  pts  = [-50 -30;50 -30;50 30;-50 30];
%             trns = [1/5 0 0 1/3 0 0; ...
%                     0.45 0 0 0.45 27.5 17.5; ...
%                     0.45 0 0 0.45 27.5 -17.5; ...
%                     0.45 0 0 0.45 -27.5 17.5; ...
%                     0.45 0 0 0.45 -27.5 -17.5];
%__________________________________________________________________________
% Input:
%     pts  ~ matice bod, inicializan polygon [x1 y1;x2 y2; ...]
%     trns ~ matice afinnch transformac ve formtu 
%               [a1 b1 c1 d1 e1 f1;
%                a2 b2 c2 d2 e2 f2;
%                ...]
%     iter ~ integer oznaujc poet iterac
%__________________________________________________________________________
% Output:
%     IFSfractal ~ pole vech polygon z posledn iterace,
%                  size(IFSfractal) ~ (length(pts),2,size(trns,1)^iter)
%__________________________________________________________________________
% 
% gen_ifs_fractal version history:
%    ver. 1.0 (22.9.2011)
%    ver. 1.1 (19.5.2014)
% 
%    Last update: 19.5.2014
% 
% Notes:
% A) size(pts,1) > 1 (at least 2 points)
% 
% B) size(trns,1) > 0 (at least 1 affine transformation)
% 
% C) iter > 0 (iteration number must be greater than 1)
% 
% Author(s): Miloslav apek, miloslav.capek@fel.cvut.cz, 2011
% 
% See also draw_fractal

    if ischar(pts) || size(pts,2) ~= 2 || size(pts,1) < 2
        error('ptsMustBenMatrixNx2withNpoints');
    end
    if ischar(trns) || size(trns,2) ~= 6
        error('trnsMustBeMatrixMx6withMafineTransformations');
    end
    if ischar(iter) || iter < 1 || mod(iter,1) ~= 0
        error('iterMustBePositiveInteger');
    end    
    
    Cell     = cell(iter+1,1);
    Cell{1}  = pts; % prvni (nulta) iterace odpovida bodum
    trnsSize = size(trns,1);

    for n = 1:iter           % po iteracich
        thisIter  = Cell{n}; % vezmu polygony z minule iterace
        newPolygs = zeros(size(thisIter,1),2,size(thisIter,3)*trnsSize);
        for m = 1:size(thisIter,3) % musime aplikovat na vsechny polygony
            thisPolyg(:,:) = thisIter(:,:,m); % po jednom si je pujcim
            for cur_tr = 1:trnsSize % a jedu opet pro vsechny transformace
                for cur_pt = 1:size(thisPolyg,1) % vsechny body polygonu
                    newPolygs(cur_pt,1,(m-1)*trnsSize+cur_tr) = ...
                        thisPolyg(cur_pt,1,:)*trns(cur_tr,1) + ...
                        thisPolyg(cur_pt,2,:)*trns(cur_tr,2) + trns(cur_tr,5);
                    newPolygs(cur_pt,2,(m-1)*trnsSize+cur_tr) = ...
                        thisPolyg(cur_pt,1,:)*trns(cur_tr,3) + ...
                        thisPolyg(cur_pt,2,:)*trns(cur_tr,4) + trns(cur_tr,6);
                end
            end
        end
        Cell{1+n} = newPolygs(:,:,:); % celou vypoc. iteraci ulozime
    end
    IFSfractal = Cell{end};  % zajima nas (ZDE!) jen posledni iterace